#include "MyGraphicsView.h"
#include "PPU.h"

// Triggers.
static GraphTrigger trigs[] = {
    { "CLK", &CLK, 100, 100 },
};

//Qt includes
#include <QGraphicsScene>
#include <QGraphicsTextItem>
#include <QTextStream>
#include <QScrollBar>
#include <QMouseEvent>
#include <QWheelEvent>
#include <QDebug>
#include <QMessageBox>

/**
* Sets up the subclassed QGraphicsView
*/
MyGraphicsView::MyGraphicsView(QWidget* parent) : QGraphicsView(parent) {

    setRenderHints(QPainter::Antialiasing | QPainter::SmoothPixmapTransform);

    //Set-up the scene
    QGraphicsScene* Scene = new QGraphicsScene(this);
    setScene(Scene);

    // load background image
    QImage image("D:/Andruxa/2C02G/2C02G_all.jpg");
    if (image.isNull()) QMessageBox::information(this, tr("Error"), tr("Cannot load background image"));

    Scene->addPixmap(QPixmap::fromImage(image));

    //Populate the scene
    for(int x = 0; x < 1000; x = x + 25) {
        for(int y = 0; y < 1000; y = y + 25) {

            if(x % 100 == 0 && y % 100 == 0) {
                Scene->addRect(x, y, 2, 2);

                QString pointString;
                QTextStream stream(&pointString);
                stream << "(" << x << "," << y << ")";
                QGraphicsTextItem* item = Scene->addText(pointString);
                item->setPos(x, y);
            } else {
                Scene->addRect(x, y, 1, 1);
            }
        }
    }

    //Set-up the view
    setSceneRect(0, 0, image.width(), image.height());

    //Use ScrollHand Drag Mode to enable Panning
    setDragMode(ScrollHandDrag);

    // Add step button.
    QPushButton *button = new QPushButton ( "Next Step", parent);
    button->move(10, 10);
    button->show();
    connect (button, SIGNAL(clicked()), this, SLOT(NextStepPressed ()) );

    // Add triggers.
    num_triggers = 0;
    int size = sizeof(trigs) / sizeof (GraphTrigger);
    MyCheckBox * check;
    for (int n=0; n<size; n++)
    {
        check = new MyCheckBox ( trigs[n].name, 0, trigs[n].link );
        Scene->addWidget (check);
        check->move( trigs[n].x, trigs[n].y );
        check->show();

        triggers[n] = check;
        num_triggers++;
    }

}

/**
  * Zoom the view in and out.
  */
void MyGraphicsView::wheelEvent(QWheelEvent* event) {

    setTransformationAnchor(QGraphicsView::AnchorUnderMouse);

    // Scale the view / do the zoom
    double scaleFactor = 1.15;
    if(event->delta() > 0) {
        // Zoom in
        scale(scaleFactor, scaleFactor);
    } else {
        // Zooming out
        scale(1.0 / scaleFactor, 1.0 / scaleFactor);
    }

    // Don't call superclass handler here
    // as wheel is normally used for moving scrollbars
}

void MyGraphicsView::NextStepPressed ()
{
    PPUStep ();

    MyCheckBox * check;
    for (int i=0; i<num_triggers; i++)
    {
        check = triggers[i];
        if ( * check->trigger_link == 0 ) check->setChecked( false );
        else check->setChecked( true );
    }
}
